package platform_user

import (
	"encoding/json"
	"fmt"
	"github.com/pkg/errors"
	"io"
	"net/http"
	"net/url"
	"strconv"
)

type MyProxy struct {
	Id         int64  `json:"id"`
	Ip         string `json:"ip"`
	Port       string `json:"port"`
	SchemeType int    `json:"scheme_type"`
	Content    string `json:"content"`
}

func RetrieveBilibili(pUid string) (*PlatformUser, error) {
	proxy := func(_ *http.Request) (*url.URL, error) {
		httpResponse, err := http.Get(fmt.Sprintf("http://127.0.0.1:9999/sql?query=%s",
			url.QueryEscape("SELECT * FROM proxy ORDER BY RAND() limit 1")))
		if err != nil {
			return nil, err
		}
		defer httpResponse.Body.Close()

		all, err := io.ReadAll(httpResponse.Body)
		if err != nil {
			return nil, err
		}
		var respStruct struct {
			Error   string    `json:"error"`
			Message []MyProxy `json:"message"`
		}
		err = json.Unmarshal(all, &respStruct)
		if err != nil {
			return nil, err
		}
		u := fmt.Sprintf("http://%s", respStruct.Message[0].Content)
		return url.Parse(u)
	}

	transport := &http.Transport{Proxy: proxy}
	client := &http.Client{Transport: transport}

	httpResponse, err := client.Get(fmt.Sprintf("http://api.bilibili.com/x/space/acc/info?mid=%s", pUid))
	if err != nil {
		return nil, errors.Wrap(err, "网络错误")
	}
	all, err := io.ReadAll(httpResponse.Body)
	if err != nil {
		return nil, errors.Wrap(err, "读取接口数据错误 IO/Error")
	}
	var resp struct {
		Code    int64                  `json:"code"`
		Message string                 `json:"message"`
		Ttl     int64                  `json:"ttl"` // 未知意义
		Data    map[string]interface{} `json:"data,omitempty"`
	}
	err = json.Unmarshal(all, &resp)
	if err != nil {
		return nil, errors.Wrap(err, "读取接口数据错误 Unmarshal/Error")
	}
	if resp.Code == 0 && resp.Data != nil {
		// 成功
		data, err := json.Marshal(resp.Data)
		if err != nil {
			return nil, errors.Wrap(err, "读取接口数据错误 Marshal/Data/Error")
		}
		return &PlatformUser{
			PUid:    strconv.FormatInt(int64(resp.Data["mid"].(float64)), 10),
			PName:   resp.Data["name"].(string),
			PAvatar: resp.Data["face"].(string),
			PInfo:   string(data),
		}, nil
	} else {
		return nil, errors.New(resp.Message)
	}
}
