package logic

import (
	"context"
	"git.noahlan.cn/northlan/ntools-go/uuid"
	"github.com/pkg/errors"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/logic/platform_user"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type RetrievePlatformUserLogic struct {
	ctx                   context.Context
	svcCtx                *svc.ServiceContext
	platformUserRetriever *platform_user.UserRetriever

	logx.Logger
}

func NewRetrievePlatformUserLogic(ctx context.Context, svcCtx *svc.ServiceContext) *RetrievePlatformUserLogic {
	return &RetrievePlatformUserLogic{
		ctx:                   ctx,
		svcCtx:                svcCtx,
		platformUserRetriever: platform_user.NewUserRetriever(ctx, svcCtx),
		Logger:                logx.WithContext(ctx),
	}
}

// RetrievePlatformUser 查询或创建用户（此时不查询平台用户信息）
func (l *RetrievePlatformUserLogic) RetrievePlatformUser(in *pb.PlatformUserReq) (*pb.PlatformUserResp, error) {
	var err error
	dbPlatformUser, err := l.svcCtx.UserPlatformModel.FindOneByPlatformAndPUid(l.ctx, in.Platform, in.PUid)
	if err != nil {
		if !errors.Is(err, model.ErrNotFound) {
			return nil, err // sql错误
		}
	}
	if dbPlatformUser != nil {
		var username string
		if one, err := l.svcCtx.UserModel.FindOne(l.ctx, dbPlatformUser.UserId); err != nil {
			username = one.Username
		}
		return &pb.PlatformUserResp{
			User: l.buildPBUser(username, dbPlatformUser),
		}, nil
	}

	// create
	dbUser := &model.User{
		Id:       uuid.NextId(),
		Username: "",
	}
	if err = l.svcCtx.UserModel.Insert(l.ctx, dbUser); err != nil {
		return nil, errors.Wrap(err, "插入用户数据失败")
	}
	dbPlatformUser = &model.UserPlatform{
		Id:       uuid.NextId(),
		UserId:   dbUser.Id,
		Platform: in.Platform,
		PInfo:    "{}",
	}
	if err = l.svcCtx.UserPlatformModel.Insert(l.ctx, dbPlatformUser); err != nil {
		return nil, errors.Wrap(err, "插入平台用户数据失败")
	}

	return &pb.PlatformUserResp{
		User: l.buildPBUser(dbUser.Username, dbPlatformUser),
	}, nil
}

func (l *RetrievePlatformUserLogic) buildPBUser(username string, dbPlatformUser *model.UserPlatform) *pb.User {
	return &pb.User{
		Id:       dbPlatformUser.UserId,
		Username: username,
		Platform: dbPlatformUser.Platform,
		PUid:     dbPlatformUser.PUid,
		PUname:   dbPlatformUser.PUname,
		PAvatar:  dbPlatformUser.PAvatar,
		PInfo:    dbPlatformUser.PInfo,
	}
}
